from typing import Any, Dict
from datetime import datetime

from google.cloud import bigquery


class NotificationStatistic:
    def __init__(self, project_id: str, bq_client: Any) -> None:
        self.project_id = project_id
        self.bq_client = bq_client

    def capture(self, data: Dict[str, Any]):
        """Capture event

        Capture event and transport to BigQuery.

        Args:
            data: a dict consists of:
                    - notification_id
                    - event_type
                    - object_id
                    - taxonomy_id
                    - created_at
                    - notification_type

        Returns:
            BigQuery error message or None
        """

        available_events = [
            "sent_to_slackbot",
            "history_written",
            "notification_requested",
        ]

        if data["event_type"] not in available_events:
            raise ValueError("Event type is not supported")

        table_id = f"{self.project_id}.notification_platform.statistics"

        try:
            errors = self.bq_client.insert_rows_json(table_id, [data])
            return errors
        except:
            raise


def capture_event(data: Dict[str, Any], is_production: bool = True):
    """Capture event

    Capture event and transport to BigQuery.

    Args:
        data: a dict consists of:
                - notification_id
                - event_type
                - object_id
                - taxonomy_id
                - created_at
                - notification_type
        is_production: A flag to select the bigquery dataset.

    Returns:
        BigQuery error message or None
    """

    project_id = "kumparan-data" if is_production else "kumparan-data-staging"

    ns = NotificationStatistic(project_id=project_id, bq_client=bigquery.Client())

    return ns.capture(data)


from setuptools import setup

with open("README.md") as f:
    readme = f.read()

setup(
    name="notifstats",
    packages=["notifstats"],
    python_requires=">=3.6",
    version="0.0.4",
    license="MIT",
    description="Notification statistic collector",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="I Putu Alfred Crosby",
    author_email="alfred.crosby@kumparan.com",
    url="https://github.com/kumparan/notification-stats",
    download_url="https://github.com/kumparan/notification-stats/archive/v_01.tar.gz",
    keywords=[
        "Notification Statistic",
        "Kumparan",
    ],
    install_requires=["google-cloud-bigquery>=1.20.0"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
)